==> payara-embedded-all-5.201-sources.jar: fish/payara/ejb/http/protocol/rs/InvokeMethodResponseDeserializer.java <==
/*
 *    DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 *    Copyright (c) [2019] Payara Foundation and/or its affiliates. All rights reserved.
 *
 *    The contents of this file are subject to the terms of either the GNU
 *    General Public License Version 2 only ("GPL") or the Common Development
 *    and Distribution License("CDDL") (collectively, the "License").  You
 *    may not use this file except in compliance with the License.  You can
 *    obtain a copy of the License at
 *    https://github.com/payara/Payara/blob/master/LICENSE.txt
 *    See the License for the specific
 *    language governing permissions and limitations under the License.
 *
 *    When distributing the software, include this License Header Notice in each
 *    file and include the License file at glassfish/legal/LICENSE.txt.
 *
 *    GPL Classpath Exception:
 *    The Payara Foundation designates this particular file as subject to the "Classpath"
 *    exception as provided by the Payara Foundation in the GPL Version 2 section of the License
 *    file that accompanied this code.
 *
 *    Modifications:
 *    If applicable, add the following below the License Header, with the fields
 *    enclosed by brackets [] replaced by your own identifying information:
 *    "Portions Copyright [year] [name of copyright owner]"
 *
 *    Contributor(s):
 *    If you wish your version of this file to be governed by only the CDDL or
 *    only the GPL Version 2, indicate your decision by adding "[Contributor]
 *    elects to include this software in this distribution under the [CDDL or GPL
 *    Version 2] license."  If you don't indicate a single choice of license, a
 *    recipient has the option to distribute your version of this file under
 *    either the CDDL, the GPL Version 2 or to extend the choice of license to
 *    its licensees as provided above.  However, if you add GPL Version 2 code
 *    and therefore, elected the GPL Version 2 license, then the option applies
 *    only if the new code is made subject to such option by the copyright
 *    holder.
 */

package fish.payara.ejb.http.protocol.rs;

import fish.payara.ejb.http.protocol.InvokeMethodResponse;

import javax.json.bind.JsonbBuilder;
import javax.json.bind.serializer.DeserializationContext;
import javax.json.bind.serializer.JsonbDeserializer;
import javax.json.stream.JsonParser;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;

public class InvokeMethodResponseDeserializer implements JsonbDeserializer<InvokeMethodResponse> {
    private final Type methodReturntype;

    public InvokeMethodResponseDeserializer(Type methodReturntype) {
        this.methodReturntype = methodReturntype;
    }

    @Override
    public InvokeMethodResponse deserialize(JsonParser parser, DeserializationContext ctx, Type rtType) {
        String type = null;
        Object result = null;
        boolean resultPresent = false;
        while (parser.hasNext()) {
            JsonParser.Event event = parser.next();
            if (event == JsonParser.Event.KEY_NAME && parser.getString().equals("type")) {
                parser.next();
                type = parser.getString();
            } else if (event == JsonParser.Event.KEY_NAME && parser.getString().equals("result")) {
                Type resultClass = determineClass(type);
                result = ctx.deserialize(resultClass, parser);
                resultPresent = true;
            }
        }
        if (!resultPresent) {
            // no result means null. But there's no way to pass that to Yasson's ctx, as it only expects its own parser
            result = JsonbBuilder.create().fromJson("null", determineClass(type));
        }
        return new InvokeMethodResponse(result);
    }

    private Type determineClass(String type) {
        // the type provided in response is only useful for non-parametrized, non-container types (and these are always
        // parametrized -- List<T>, Map<K,V>, ...)
        if (this.methodReturntype instanceof ParameterizedType || type == null) {
            return this.methodReturntype;
        }
        try {
            return Class.forName(type);
        } catch (ClassNotFoundException e) {
            return this.methodReturntype;
        }
    }
}

==> payara-embedded-all-6.2023.12-sources.jar: fish/payara/ejb/http/protocol/rs/InvokeMethodResponseDeserializer.java <==
/*
 *    DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 *    Copyright (c) [2019-2021] Payara Foundation and/or its affiliates. All rights reserved.
 *
 *    The contents of this file are subject to the terms of either the GNU
 *    General Public License Version 2 only ("GPL") or the Common Development
 *    and Distribution License("CDDL") (collectively, the "License").  You
 *    may not use this file except in compliance with the License.  You can
 *    obtain a copy of the License at
 *    https://github.com/payara/Payara/blob/master/LICENSE.txt
 *    See the License for the specific
 *    language governing permissions and limitations under the License.
 *
 *    When distributing the software, include this License Header Notice in each
 *    file and include the License file at glassfish/legal/LICENSE.txt.
 *
 *    GPL Classpath Exception:
 *    The Payara Foundation designates this particular file as subject to the "Classpath"
 *    exception as provided by the Payara Foundation in the GPL Version 2 section of the License
 *    file that accompanied this code.
 *
 *    Modifications:
 *    If applicable, add the following below the License Header, with the fields
 *    enclosed by brackets [] replaced by your own identifying information:
 *    "Portions Copyright [year] [name of copyright owner]"
 *
 *    Contributor(s):
 *    If you wish your version of this file to be governed by only the CDDL or
 *    only the GPL Version 2, indicate your decision by adding "[Contributor]
 *    elects to include this software in this distribution under the [CDDL or GPL
 *    Version 2] license."  If you don't indicate a single choice of license, a
 *    recipient has the option to distribute your version of this file under
 *    either the CDDL, the GPL Version 2 or to extend the choice of license to
 *    its licensees as provided above.  However, if you add GPL Version 2 code
 *    and therefore, elected the GPL Version 2 license, then the option applies
 *    only if the new code is made subject to such option by the copyright
 *    holder.
 */

package fish.payara.ejb.http.protocol.rs;

import fish.payara.ejb.http.protocol.InvokeMethodResponse;

import jakarta.json.bind.JsonbBuilder;
import jakarta.json.bind.serializer.DeserializationContext;
import jakarta.json.bind.serializer.JsonbDeserializer;
import jakarta.json.stream.JsonParser;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;

public class InvokeMethodResponseDeserializer implements JsonbDeserializer<InvokeMethodResponse> {
    private final Type methodReturntype;

    public InvokeMethodResponseDeserializer(Type methodReturntype) {
        this.methodReturntype = methodReturntype;
    }

    @Override
    public InvokeMethodResponse deserialize(JsonParser parser, DeserializationContext ctx, Type rtType) {
        String type = null;
        Object result = null;
        boolean resultPresent = false;
        while (parser.hasNext()) {
            JsonParser.Event event = parser.next();
            if (event == JsonParser.Event.KEY_NAME && parser.getString().equals("type")) {
                parser.next();
                type = parser.getString();
            } else if (event == JsonParser.Event.KEY_NAME && parser.getString().equals("result")) {
                Type resultClass = determineClass(type);
                parser.next();
                result = ctx.deserialize(resultClass, parser);
                resultPresent = true;
            }
        }
        if (!resultPresent) {
            // no result means null. But there's no way to pass that to Yasson's ctx, as it only expects its own parser
            result = JsonbBuilder.create().fromJson("null", determineClass(type));
        }
        return new InvokeMethodResponse(result);
    }

    private Type determineClass(String type) {
        // the type provided in response is only useful for non-parametrized, non-container types (and these are always
        // parametrized -- List<T>, Map<K,V>, ...)
        if (this.methodReturntype instanceof ParameterizedType || type == null) {
            return this.methodReturntype;
        }
        try {
            return Class.forName(type);
        } catch (ClassNotFoundException e) {
            return this.methodReturntype;
        }
    }
}
